﻿/// Copyright © 2010-2011 Sebastian Thies
/// This program is distributed under the terms of the GNU General Public License
/// http://www.gnu.org/licenses/#GPL
/// email: sebastian.thies@arcor.de
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Collections.Specialized;
using System.IO;
using System.Windows.Forms;
using ThiesMediaProject.ToolForms;
using System.Diagnostics;
using ThiesMediaProject.Properties;

namespace ThiesMediaProject.General
{
    /// <summary>
    /// List of Filmobjects and List or global functions
    /// </summary>
    public class Films : BindingList<Film>
    {
        public Film this[string folder]    // Indexer declaration
        {
            get
            {
                foreach (Film film in this)
                {
                    if (film.Path == folder)
                    {
                        return film;    
                    }
                }
                var film2 = new Film(GetNewFilmId());
                return film2; 
            }
            
        }


        public StringCollection GlobalFilmPaths { get; set; }

        ListSortDirection LastSortDirection = ListSortDirection.Ascending;
        bool IsSorted = false;
        Random RandomIDGen = new Random();

        /// <summary>
        /// empty Constructor
        /// </summary>
        public Films()
        {

        }


        /// <summary>
        /// Constructor for adding films automaticly
        /// </summary>
        /// <param name="globalFilmPaths">list of global film directories</param>
        public Films(StringCollection globalFilmPaths, bool showDialog)
        {
            GlobalFilmPaths = globalFilmPaths;
            GetFilmsFromGlobalFilmPaths(showDialog);
        }


        /// <summary>
        /// for each path of directory found in global path a film item will be added
        /// </summary>
        public void GetFilmsFromGlobalFilmPaths(bool showDialog)
        {
            if (GlobalFilmPaths != null)
            {
                var sf = new StatusForm("Loading Films");
                if (showDialog)
                {
                    sf.MaxValue = GlobalFilmPaths.Count;
                    sf.Show();
                }

                if (GlobalFilmPaths != null)
                {
                    foreach (string globalPath in GlobalFilmPaths)
                    {
                        if (showDialog)
                            sf.Value++;
                        if (Directory.Exists(globalPath))
                        {
                            string[] filmFolders = Directory.GetDirectories(globalPath);
                            foreach (string filmFolder in filmFolders)
                            {
                                if (
                                    !(new DirectoryInfo(filmFolder).Attributes.HasFlag(FileAttributes.System)) &&
                                    !(new DirectoryInfo(filmFolder).Attributes.HasFlag(FileAttributes.ReadOnly)) &&
                                    !(new DirectoryInfo(filmFolder).Attributes.HasFlag(FileAttributes.Hidden))
                                    )
                                {
                                    var film = new Film(filmFolder, GetNewFilmId());
                                    this.Add(film);
                                }
                            }
                        }
                    }
                }

                if (Settings.Default.DeletedMovies != null)
                {
                    foreach (string item in Settings.Default.DeletedMovies)
                    {
                        try
                        {
                            this.Remove(this.GetFilmByPath(item));
                        }
                        catch (Exception){}                        
                    }
                }


                if (showDialog)
                {
                    sf.Value = 0;
                    sf.Close();
                }
            }
        }       

        public string GetNewFilmId()
        {
            var dateTime = DateTime.Now;
            string newID = "";
            do
            {
                newID = dateTime.Year.ToString();
                newID = newID + dateTime.Month.ToString("00");
                newID = newID + dateTime.Day.ToString("00");
                newID = newID + dateTime.Hour.ToString("00");
                newID = newID + dateTime.Minute.ToString("00");
                newID = newID + dateTime.Second.ToString("00");
                newID = newID + dateTime.Millisecond.ToString("0000");
                newID = newID + RandomIDGen.Next(999999999).ToString("000000000");
                Application.DoEvents();
            } while (ExistingID(newID));
            
            return newID;
        }

        private bool ExistingID(string filmId)
        {
            if (Data.DataFilms != null)
            {
                foreach (Film item in Data.DataFilms)
                {
                    if (item.FilmID == filmId)
                    {
                        return true;
                    }
                }
            }
            return false;
        }


        public void SaveAllXml()
        {
            foreach (Film film in this)
            {
                film.SaveXML();
            }
        }

        public void LoadAllXml()
        {
            foreach (Film film in this)
            {
                film.LoadXML();
            }
        }

        public bool ContainsId(string filmId)
        {
            foreach (Film item in this)
            {
                if (filmId == item.FilmID)
                {
                    return true;
                }
            }
            return false;
        }

        public Film GetFilmById(string filmId)
        {
            foreach (Film item in this)
            {
                if (filmId == item.FilmID)
                {
                    return item;
                }
            }
            return null;
        }

        public Film GetFilmByPath(string filmPath)
        {
            foreach (Film item in this)
            {
                if (filmPath == item.Path)
                {
                    return item;
                }
            }

            while (new DirectoryInfo(filmPath).Parent != null)
            {
                DirectoryInfo drInfo = new DirectoryInfo(filmPath);
                filmPath = drInfo.Parent.FullName;
                foreach (Film item in this)
                {
                    if (filmPath == item.Path)
                    {
                        return item;
                    }
                }
            }
            

            return null;
        }


        protected override bool SupportsSortingCore
        {
            get
            {
                return true;
            }
        }

        public void Sort(PropertyDescriptor prop, ListSortDirection direction)
        {
            ApplySortCore(prop, direction);
        }

        protected override void ApplySortCore(PropertyDescriptor prop, ListSortDirection direction)
        {
            foreach (Film film in this)
            {
                film.SortingProperty = prop.GetValue(film);
            }
            IEnumerable<Film> query;

            if (direction == ListSortDirection.Ascending)
            {
                query = this.OrderBy(film => film.SortingProperty);
            }
            else
            {
                query = this.OrderByDescending(film => film.SortingProperty);
            }

            int i = 0;
            foreach (Film film in query)
            {
                this.Items[i] = film;
                i++;
            }
            IsSorted = true;
        }

        protected override void RemoveSortCore()
        {
            
        }

        protected override bool IsSortedCore
        {
            get
            {
                return IsSorted;
            }
        }

        protected override ListSortDirection SortDirectionCore
        {
            get
            {
                return LastSortDirection;
            }
        }
    }
}
